package RedCorona.LongTasks {
	import RedCorona.Events.DataEvent;
	import flash.events.EventDispatcher;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.utils.getTimer;
	import RedCorona.Events.NumericEvent;
	
	/**
	* Allows long, multi-step tasks to be run over several frames
	* Tasks pushed onto the queue won't be started until the next frame
	*/
	public class TaskManager extends EventDispatcher {
		private var _stage:Stage;
		private var queue:Vector.<ITask> = new Vector.<ITask>();
		private var currentTask:ITask = null;
		private var maxTime:int;
		
		public static const TASK_COMPLETE_EVENT:String = "taskComplete",
			QUEUE_COMPLETE_EVENT:String = "queueComplete",
			QUEUE_PROGRESS_EVENT:String = "queueProgress",
			TASK_PROGRESS_EVENT:String = "taskProgress";
		
		public function set stage(stage:Stage) : void {
			if (_stage) _stage.removeEventListener(Event.ENTER_FRAME, updateQueue);
			_stage = stage;
			if (_stage) _stage.addEventListener(Event.ENTER_FRAME, updateQueue);
		}
		
		public function TaskManager(maxTime:int = 20, stage:Stage = null) {
			this.stage = stage;
			this.maxTime = maxTime;
		}
		
		public function add(task: ITask) : void {
			if (currentTask == null) currentTask = task;
			else queue.push(task);
		}
		
		private function updateQueue(e:Event) : void {
			if (currentTask == null) return;
			
			var time:int = getTimer();
			do {
				currentTask.doNextItem();
				if (currentTask.complete) {
					dispatchEvent(new DataEvent(TASK_COMPLETE_EVENT, currentTask));
					if (queue.length > 0) {
						dispatchEvent(new NumericEvent(QUEUE_PROGRESS_EVENT, queue.length));
						currentTask = queue.splice(0, 1)[0];
					} else {
						dispatchEvent(new Event(QUEUE_COMPLETE_EVENT));
						currentTask = null;
					}
				} else {
					dispatchEvent(new DataEvent(TASK_PROGRESS_EVENT, currentTask));
				}
			} while (currentTask && (maxTime > getTimer() - time));
		}
	}
}